﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;

namespace ErrorHandlingInWF
{
    class Program
    {
        static WorkflowRuntime workflowRuntime;

        static void Main(string[] args)
        {
            using(workflowRuntime = new WorkflowRuntime())
            {
                AutoResetEvent waitHandle = new AutoResetEvent(false);
                workflowRuntime.WorkflowCompleted += delegate(object sender, WorkflowCompletedEventArgs e) {waitHandle.Set();};
                
                //capture terminated workflows and get notified of the exception
                workflowRuntime.WorkflowTerminated += delegate(object sender, WorkflowTerminatedEventArgs e)
                {
                    Console.WriteLine(e.Exception.Message);
                    waitHandle.Set();
                };

                //Get notified of exceptions occuring in the runtime services
                workflowRuntime.ServicesExceptionNotHandled += delegate(object sender, ServicesExceptionNotHandledEventArgs snhe)
                {
                    Console.WriteLine(snhe.Exception.Message);
                };

                Dictionary<string, Action> actions = new Dictionary<string, Action>();
                actions.Add("Manual Retry", RunManualRetry);
                actions.Add("Auto Retry", RunRetry);
                actions.Add("Error handling", RunErrorHandling);

                ConsoleMenu menu = new ConsoleMenu(actions);
                menu.Run();
                

                waitHandle.WaitOne();
            }


        }

        private static void RunManualRetry()
        {
            WorkflowInstance instance =
                workflowRuntime.CreateWorkflow(
                typeof(ErrorHandlingInWF.ManualRetryLogic));
            instance.Start();
        }

        private static void RunRetry()
        {
            WorkflowInstance instance =
                workflowRuntime.CreateWorkflow(
                typeof(ErrorHandlingInWF.WorkflowWithRetryActivity));
            instance.Start();
        }

        private static void RunErrorHandling( )
        {
            WorkflowInstance instance =
                workflowRuntime.CreateWorkflow(
                typeof(ErrorHandlingInWF.Workflow1));
            instance.Start();
        }
        
    }
}
